REDIRECT_STATUS_CODES = {301, 302, 303, 307, 308}
RESERVED_KWARGS = {"state", "headers", "cookies", "request", "socket", "data", "query"}
EXTRA_KEY_REQUIRED = "required"
EXTRA_KEY_VALUE_TYPE = "value_type"
EXTRA_KEY_IS_PARAMETER = "is_parameter"
EXTRA_KEY_IS_DEPENDENCY = "is_dependency"
EXTRA_KEY_SKIP_VALIDATION = "skip_validation"
